@@ -16,15 +16,24 @@ devtools::install(pkg = "C:/Users/hheiling/Documents/GitHub/glmmPen")
 devtools::check(pkg = "C:/Users/hheiling/Documents/GitHub/glmmPen")
 
 # Build package
+devtools::build(pkg = "C:/Users/hheiling/Documents/GitHub/glmmPen",
+                path = "C:/Users/hheiling/Documents/GitHub/CRAN")
 # devtools::build(pkg = "C:/Users/hheiling/Documents/GitHub/glmmPen",
 #                 path = "C:/Users/hheiling/Documents/Longleaf")
-devtools::build(pkg = "C:/Users/hheiling/Documents/GitHub/glmmPen",
-                path = "C:/Users/hheiling/Documents/LaTeX/glmmPen JSS paper")
+# devtools::build(pkg = "C:/Users/hheiling/Documents/GitHub/glmmPen",
+#                 path = "C:/Users/hheiling/Documents/LaTeX/glmmPen JSS paper")
 
 # Build manual
 devtools::build_manual(pkg = "C:/Users/hheiling/Documents/GitHub/glmmPen", 
                        path = "C:/Users/hheiling/Documents/GitHub/glmmPen/inst")
 
+# Versions:
+## 1.5.2.6: Added thresholding checks for Poisson family variable selection
+## 1.5.2.5: Removed Z_standardized option, added standardization = T/F option (X and Z
+##    either both standardized or both non-standardized)
+## 1.5.2.4: Added alternative convergence options
+## 1.5.2.3: Implemented proximal line search algorithm
+
 # other branch
 # install_github("hheiling/glmmPen", ref = "alt_branch", force = TRUE)
 # master branch
@@ -46,6 +55,7 @@ devtools::build_manual(pkg = "C:/Users/hheiling/Documents/GitHub/glmmPen",
 
 library(rstantools)
 # use_rstan(pkgdir = "C:/Users/hheiling/Documents/GitHub/glmmPen")
+# Creates or update package-specific system files to compile .stan model files found in inst/stan.
 rstan_config(pkgdir = "C:/Users/hheiling/Documents/GitHub/glmmPen")
 
 
