@@ -33,9 +33,9 @@ arma::vec resid_nu_i(double yi, arma::vec eta, const char* family, int link, dou
   // If not binomial or gaussian family with canonical link, calculate nu = max(weights)
   // non-canonical link options not yet available
   if((std::strcmp(family,pois) == 0) && (link == 20)){
-    
+
     weights = varfun(family, mu, phi);
-    
+
     for(m=0;m<M;m++){
       if(mu_check(m)==0){
         weights(m) = 0.0;
@@ -45,13 +45,13 @@ arma::vec resid_nu_i(double yi, arma::vec eta, const char* family, int link, dou
     if(nu_tmp > nu){
       nu = nu_tmp;
     }
-    
+
     if(nu < 0.0001){
       nu = 0.0001;
     }
   } // End family if-else
   
-  resid = ((yi*const_ones) - mu); // Will divide zetaj by nu later
+  resid = ((yi*const_ones) - mu); // Will divide total zetaj by nu later
   for(m=0;m<M;m++){
     if(mu_check(m)==0){
       resid(m) = 0.0; // Ignore invalid mu
@@ -65,6 +65,33 @@ arma::vec resid_nu_i(double yi, arma::vec eta, const char* family, int link, dou
   return(output);
 }
 
+// Calculates the M residuals for an individual given eta
+// Assumes update to nu occurs elsewhere
+// Used in "grp_CD_XZ_step.cpp" code
+arma::vec resid_i(double yi, arma::vec eta, const char* family, int link){
+  
+  int M = eta.n_elem;
+  int m = 0;
+  
+  arma::vec mu(M);
+  arma::vec mu_check(M);
+  arma::vec resid(M);
+  arma::vec const_ones(M); const_ones.ones();
+  
+  // Update mu and resid
+  mu = invlink(link, eta);
+  mu_check = muvalid(family, mu);
+  
+  resid = ((yi*const_ones) - mu); // Will divide by nu later
+  for(m=0;m<M;m++){
+    if(mu_check(m)==0){
+      resid(m) = 0.0; // Ignore invalid mu
+    }
+  }
+  
+  return(resid);
+}
+
 
 // Calculates fixed effects zetaj using residuals matrix
 arma::vec zeta_fixef_calc(arma::mat X, arma::mat resid, arma::uvec idxj){
@@ -87,65 +114,71 @@ arma::vec zeta_fixef_calc(arma::mat X, arma::mat resid, arma::uvec idxj){
   return(zetaj);
 }
 
+// Quadratic approximation to Q function estimate (based on Taylor series expansion about 
+//   the previous beta0 coefficient estimates)
+// Note: in M-step, have already calculated Q evaluated at past coefficient value (Q0)
+// Note: nu = 1 / step_size
+// [[Rcpp::export]]
+double Qfun_quad_beta(double Q0, double step_size, const arma::mat& diff0,
+                      const arma::mat& eta, const arma::mat& eta0,
+                      const arma::vec& beta, const arma::vec& beta0){
+  
+  int N = eta.n_cols;
+  int M = eta.n_rows;
+  int p_tot = beta.n_elem;
+  
+  int i = 0;
+  
+  double Q_quad = 0;
+  double term1 = 0;
+  double term2 = 0;
+  
+  arma::vec eta_diff(M);
+  arma::vec beta_diff(p_tot);
+  
+  for(i=0;i<N;i++){
+    eta_diff = eta.col(i) - eta0.col(i);
+    term1 = term1 + sum(diff0.col(i) % eta_diff);
+  }
+  
+  beta_diff = beta - beta0;
+  term2 = sum(beta_diff % beta_diff);
+  
+  Q_quad = Q0 - term1 / M + (0.5 * N / step_size) * term2 ; // 0.5 * final term?
+  
+  return(Q_quad);
+  
+}
 
-
-// Non-canconical link options
-// arma::vec resid_nu_i(double yi, arma::vec eta, const char* family, int link, double nu, double phi){
+// // Quadratic approximation to Q function estimate (based on Taylor series expansion about 
+// //    linear predictor evaluated at previous beta0 coefficient estimates)
+// // Note: in M-step, have already calculated Q evaluated at past coefficient value (Q0)
+// // Note: nu = 1 / step_size
+// // [[Rcpp::export]]
+// double Qfun_quad(double Q0, double nu, 
+//                  const arma::mat& diff0, const arma::mat& eta, const arma::mat& eta0){
 //   
-//   int M = eta.n_elem;
-//   int m = 0;
+//   int N = eta.n_cols;
+//   int M = eta.n_rows;
 //   
-//   arma::vec mu(M);
-//   arma::vec mu_check(M);
-//   arma::vec deriv(M);
-//   arma::vec Vmu(M);
-//   arma::vec resid(M);
-//   arma::vec weights(M);
-//   arma::vec const_ones(M); const_ones.ones();
+//   int i = 0;
 //   
-//   const char* bin = "binomial";
-//   const char* gaus = "gaussian";
+//   double Q_quad = 0;
+//   double term1 = 0;
+//   double term2 = 0;
 //   
-//   double nu_tmp = 0.0;
+//   arma::vec eta_diff(M);
 //   
-//   arma::vec output(M+1);
-//   
-//   // Update mu, resid, weights
-//   mu = invlink(link, eta);
-//   mu_check = muvalid(family, mu);
-//   
-//   // If not binomial or gaussian family with canonical link, calculate nu = max(weights)
-//   if(!((std::strcmp(family, bin) == 0) & (link == 10)) & !((std::strcmp(family, gaus) == 0) & (link == 30))){
-//     
-//     deriv = dlink(link, mu);
-//     Vmu = varfun(family, mu, phi);
-//     
-//     weights = const_ones / (deriv % deriv % Vmu);
-//     for(m=0;m<M;m++){
-//       if(mu_check(m)==0){
-//         weights(m) = 0.0;
-//       }
-//     }
-//     nu_tmp = max(weights);
-//     if(nu_tmp > nu){
-//       nu = nu_tmp;
-//     }
-//   } // End family if-else
-//   
-//   resid = ((yi*const_ones) - mu); // Will divide zetaj by nu later
-//   for(m=0;m<M;m++){
-//     if(mu_check(m)==0){
-//       resid(m) = 0.0; // Ignore invalid mu
-//     }
+//   for(i=0;i<N;i++){
+//     eta_diff = eta.col(i) - eta0.col(i);
+//     term1 = term1 + sum(diff0.col(i) % eta_diff);
+//     term2 = term2 + sum(eta_diff % eta_diff);
 //   }
 //   
+//   Q_quad = Q0 - term1 / M + (0.5 * nu) * term2 / M;
 //   
-//   output.subvec(0,M-1) = resid;
-//   output(M) = nu;
+//   return(Q_quad);
 //   
-//   return(output);
 // }
 
-
-
 /////////////////////////////////////////////////////////////////////////////////////////////////
